@@ -34,34 +34,37 @@ using namespace fcnn::internal;
 template <typename T>
 void
 fcnn::internal::feedf(const int *lays, int no_lays, const int *n_pts,
-                      const T *w_val, int hl_af, T hl_af_p, int ol_af, T ol_af_p,
+                      const T *w_val, const int *af, const T *af_p,
                       T *n_st)
 {
     int wi = 0, ni = n_pts[1], l = 1, L = no_lays - 1;
+    int actf; T actfp;
     // hidden layers
     for (; l < L; ++l) {
         int npl = lays[l - 1], nn = n_pts[l + 1];
         T *nplptr = &n_st[n_pts[l - 1]];
+        actf = af[l]; actfp = af_p[l];
         for (; ni < nn; ++ni, wi += npl) {
             // bias
             T d = w_val[wi++];
             // dot product
             d += dot(npl, nplptr, 1, w_val + wi, 1);
             // activation
-            n_st[ni] = mlp_act_f(hl_af, hl_af_p, d);
+            n_st[ni] = mlp_act_f(actf, actfp, d);
         }
     }
 
     // output layer
     int npl = lays[l - 1], nn = n_pts[l + 1];
     T *nplptr = &n_st[n_pts[l - 1]];
+    actf = af[l]; actfp = af_p[l];
     for (; ni < nn; ++ni, wi += npl) {
         // bias
         T d = w_val[wi++];
         // dot product
         d += dot(npl, nplptr, 1, w_val + wi, 1);
         // activation
-        n_st[ni] = mlp_act_f(ol_af, ol_af_p, d);
+        n_st[ni] = mlp_act_f(actf, actfp, d);
     }
 }
 
@@ -71,26 +74,19 @@ template <typename T>
 void
 fcnn::internal::backprop(const int *lays, int no_lays, const int *n_pts,
                          int no_weights, const T *w_val,
-                         int hl_af, T hl_af_p, int ol_af, T ol_af_p,
+                         const int *af, const T *af_p,
                          const T *n_st, T *delta, T *grad)
 {
     // initialisation
     int l = no_lays - 1, ni = n_pts[no_lays] - 1, wi = no_weights, nlpl;
     register T d;
-    // output layer deltas
-    nlpl = lays[l - 1];
-    for (int nl = lays[l]; nl; --nl, --ni) {
-        d = delta[ni] * mlp_act_f_der(ol_af, ol_af_p, n_st[ni]);
-        wi -= nlpl;
-        axpy(nlpl, d, w_val + wi, 1, delta + n_pts[l - 1], 1);
-        axpy(nlpl, d, n_st + n_pts[l - 1], 1, grad + wi, 1);
-        grad[--wi] += d;
-    }
-    // hidden layers except for the 1st
-    for (--l; l > 1; --l) {
+    int actf; T actfp;
+    // output and hidden layers except for the 1st
+    for (; l > 1; --l) {
         nlpl = lays[l - 1];
+        actf = af[l]; actfp = af_p[l];
         for (int nl = lays[l]; nl; --nl, --ni) {
-            d = delta[ni] * mlp_act_f_der(hl_af, hl_af_p, n_st[ni]);
+            d = delta[ni] * mlp_act_f_der(actf, actfp, n_st[ni]);
             wi -= nlpl;
             axpy(nlpl, d, w_val + wi, 1, delta + n_pts[l - 1], 1);
             axpy(nlpl, d, n_st + n_pts[l - 1], 1, grad + wi, 1);
@@ -99,8 +95,9 @@ fcnn::internal::backprop(const int *lays, int no_lays, const int *n_pts,
     }
     // first hidden layer
     nlpl = lays[0];
+    actf = af[l]; actfp = af_p[l];
     for (int nl = lays[l]; nl; --nl, --ni) {
-        d = delta[ni] * mlp_act_f_der(hl_af, hl_af_p, n_st[ni]);
+        d = delta[ni] * mlp_act_f_der(actf, actfp, n_st[ni]);
         wi -= nlpl;
         axpy(nlpl, d, n_st + n_pts[l - 1], 1, grad + wi, 1);
         grad[--wi] += d;
@@ -113,15 +110,17 @@ template <typename T>
 void
 fcnn::internal::backpropj(const int *lays, int no_lays, const int *n_pts, int j,
                           const int *w_pts, const T *w_val,
-                          int hl_af, T hl_af_p, int ol_af, T ol_af_p,
+                          const int *af, const T *af_p,
                           const T *n_st, T *delta, T *grad)
 {
     // initialisation
     int l = no_lays - 1, ni = n_pts[l] + j,
         wi = w_pts[l] + j * (1 + lays[l - 1]) + 1, nlpl;
     register T d;
+    int actf; T actfp;
     // jth output neuron delta
-    d = delta[ni] * mlp_act_f_der(ol_af, ol_af_p, n_st[ni]);
+    actf = af[l]; actfp = af_p[l];
+    d = delta[ni] * mlp_act_f_der(actf, actfp, n_st[ni]);
     nlpl = lays[l - 1];
     axpy(nlpl, d, w_val + wi, 1, delta + n_pts[l - 1], 1);
     axpy(nlpl, d, n_st + n_pts[l - 1], 1, grad + wi, 1);
@@ -131,8 +130,9 @@ fcnn::internal::backpropj(const int *lays, int no_lays, const int *n_pts, int j,
     ni = n_pts[l] - 1;
     for (--l; l > 1; --l) {
         nlpl = lays[l - 1];
+        actf = af[l]; actfp = af_p[l];
         for (int nl = lays[l]; nl; --nl, --ni) {
-            d = delta[ni] * mlp_act_f_der(hl_af, hl_af_p, n_st[ni]);
+            d = delta[ni] * mlp_act_f_der(actf, actfp, n_st[ni]);
             wi -= nlpl;
             axpy(nlpl, d, w_val + wi, 1, delta + n_pts[l - 1], 1);
             axpy(nlpl, d, n_st + n_pts[l - 1], 1, grad + wi, 1);
@@ -141,8 +141,9 @@ fcnn::internal::backpropj(const int *lays, int no_lays, const int *n_pts, int j,
     }
     // first hidden layer
     nlpl = lays[0];
+    actf = af[l]; actfp = af_p[l];
     for (int nl = lays[l]; nl; --nl, --ni) {
-        d = delta[ni] * mlp_act_f_der(hl_af, hl_af_p, n_st[ni]);
+        d = delta[ni] * mlp_act_f_der(actf, actfp, n_st[ni]);
         wi -= nlpl;
         axpy(nlpl, d, n_st + n_pts[l - 1], 1, grad + wi, 1);
         grad[--wi] += d;
@@ -155,15 +156,17 @@ template <typename T>
 void
 fcnn::internal::backpropjd(const int *lays, int no_lays, const int *n_pts, int j,
                            const int *w_pts, const T *w_val,
-                           int hl_af, T hl_af_p, int ol_af, T ol_af_p,
+                           const int *af, const T *af_p,
                            const T *n_st, T *delta)
 {
     // initialisation
     int l = no_lays - 1, ni = n_pts[l] + j,
         wi = w_pts[l] + j * (1 + lays[l - 1]) + 1, nlpl;
     register T d;
+    int actf; T actfp;
     // jth output neuron delta
-    d = delta[ni] * mlp_act_f_der(ol_af, ol_af_p, n_st[ni]);
+    actf = af[l]; actfp = af_p[l];
+    d = delta[ni] * mlp_act_f_der(actf, actfp, n_st[ni]);
     nlpl = lays[l - 1];
     axpy(nlpl, d, w_val + wi, 1, delta + n_pts[l - 1], 1);
     // hidden layers
@@ -171,8 +174,9 @@ fcnn::internal::backpropjd(const int *lays, int no_lays, const int *n_pts, int j
     ni = n_pts[l] - 1;
     for (--l; l; --l) {
         nlpl = lays[l - 1];
+        actf = af[l]; actfp = af_p[l];
         for (int nl = lays[l]; nl; --nl, --ni) {
-            d = delta[ni] * mlp_act_f_der(hl_af, hl_af_p, n_st[ni]);
+            d = delta[ni] * mlp_act_f_der(actf, actfp, n_st[ni]);
             wi -= nlpl;
             axpy(nlpl, d, w_val + wi, 1, delta + n_pts[l - 1], 1);
             --wi;
@@ -185,28 +189,28 @@ fcnn::internal::backpropjd(const int *lays, int no_lays, const int *n_pts, int j
 // Explicit instantiations
 #ifndef FCNN_DOUBLE_ONLY
 template void fcnn::internal::feedf(const int*, int, const int*,
-                                    const float*, int, float, int, float,
+                                    const float*, const int*, const float*,
                                     float*);
 template void fcnn::internal::backprop(const int*, int, const int*,
-                                       int, const float*, int, float, int, float,
+                                       int, const float*, const int*, const float*,
                                        const float*, float*, float*);
 template void fcnn::internal::backpropj(const int*, int, const int*, int,
-                                        const int*, const float*, int, float, int, float,
+                                        const int*, const float*, const int*, const float*,
                                         const float*, float*, float*);
 template void fcnn::internal::backpropjd(const int*, int, const int*, int,
-                                         const int*, const float*, int, float, int, float,
+                                         const int*, const float*, const int*, const float*,
                                          const float*, float*);
 #endif /* FCNN_DOUBLE_ONLY */
 template void fcnn::internal::feedf(const int*, int, const int*,
-                                    const double*, int, double, int, double,
+                                    const double*, const int*, const double*,
                                     double*);
 template void fcnn::internal::backprop(const int*, int, const int*,
-                                       int, const double*, int, double, int, double,
+                                       int, const double*, const int*, const double*,
                                        const double*, double*, double*);
 template void fcnn::internal::backpropj(const int*, int, const int*, int,
-                                        const int*, const double*, int, double, int, double,
+                                        const int*, const double*, const int*, const double*,
                                         const double*, double*, double*);
 template void fcnn::internal::backpropjd(const int*, int, const int*, int,
-                                         const int*, const double*, int, double, int, double,
+                                         const int*, const double*, const int*, const double*,
                                          const double*, double*);
 
